@@ -6,6 +6,7 @@
 {-# LANGUAGE FlexibleInstances #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE EmptyDataDecls #-}
+{-# LANGUAGE PatternGuards #-}
 {-# OPTIONS_GHC -fno-warn-missing-fields #-}
 module Text.Hamlet
     ( -- * Plain HTML
@@ -28,6 +29,7 @@ module Text.Hamlet
     , ToAttributes (..)
       -- * Internal, for making more
     , HamletSettings (..)
+    , NewlineStyle (..)
     , hamletWithSettings
     , hamletFileWithSettings
     , defaultHamletSettings
@@ -35,6 +37,7 @@ module Text.Hamlet
     , Env (..)
     , HamletRules (..)
     , hamletRules
+    , ihamletRules
     , htmlRules
     ) where
 
@@ -46,12 +49,8 @@ import Data.Char (isUpper, isDigit)
 import Data.Maybe (fromMaybe)
 import Data.Text (Text, pack)
 import qualified Data.Text.Lazy as TL
-#if MIN_VERSION_blaze_html(0,5,0)
 import Text.Blaze.Html (Html, toHtml)
 import Text.Blaze.Internal (preEscapedText)
-#else
-import Text.Blaze (Html, preEscapedText, toHtml)
-#endif
 import qualified Data.Foldable as F
 import Control.Monad (mplus)
 import Data.Monoid (mempty, mappend)
@@ -166,9 +165,15 @@ docToExp env hr scope (DocCase deref cases) = do
     matches <- mapM toMatch cases
     return $ CaseE exp_ matches
   where
+    readMay s =
+        case reads s of
+            (x, ""):_ -> Just x
+            _ -> Nothing
     toMatch (idents, inside) = do
         let pat = case map unIdent idents of
                     ["_"] -> WildP
+                    [str]
+                        | Just i <- readMay str -> LitP $ IntegerL i
                     strs -> let (constr:fields) = map mkName strs
                             in ConP constr (map VarP fields)
         insideExp <- docsToExp env hr scope inside
@@ -218,14 +223,12 @@ htmlRules = do
 hamlet :: QuasiQuoter
 hamlet = hamletWithSettings hamletRules defaultHamletSettings
 
--- | A variant which adds newlines to the output. Useful for debugging
--- but may alter browser page layout.
-hamlet' :: QuasiQuoter
-hamlet' = hamletWithSettings hamletRules defaultHamletSettings{hamletNewlines=True}
-         
 xhamlet :: QuasiQuoter
 xhamlet = hamletWithSettings hamletRules xhtmlHamletSettings
 
+asHtmlUrl :: HtmlUrl url -> HtmlUrl url
+asHtmlUrl = id
+
 hamletRules :: Q HamletRules
 hamletRules = do
     i <- [|id|]
@@ -239,8 +242,9 @@ hamletRules = do
             return $ LamE [VarP r] h
     return $ HamletRules i ur em
   where
-    em (Env (Just urender) Nothing) e =
-            urender $ \ur' -> return (e `AppE` ur')
+    em (Env (Just urender) Nothing) e = do
+        asHtmlUrl' <- [|asHtmlUrl|]
+        urender $ \ur' -> return ((asHtmlUrl' `AppE` e) `AppE` ur')
     em _ _ = error "bad Env"
 
 ihamlet :: QuasiQuoter
@@ -286,7 +290,7 @@ hamletFromString qhr set s = do
     hr <- qhr
     case parseDoc set s of
         Error s' -> error s'
-        Ok d -> hrWithEnv hr $ \env -> docsToExp env hr [] d
+        Ok (mnl, d) -> hrWithEnv hr $ \env -> docsToExp env hr [] d
 
 hamletFileWithSettings :: Q HamletRules -> HamletSettings -> FilePath -> Q Exp
 hamletFileWithSettings qhr set fp = do
