@@ -1,10 +1,9 @@
 {-# LANGUAGE QuasiQuotes #-}
 {-# LANGUAGE TemplateHaskell #-}
-module HamletTest (specs) where 
+module HamletTest (spec) where
 
 import Test.HUnit hiding (Test)
 import Test.Hspec
-import Test.Hspec.HUnit
 
 import Prelude hiding (reverse)
 import Text.Hamlet
@@ -14,76 +13,79 @@ import qualified Data.List
 import qualified Data.List as L
 import qualified Data.Map as Map
 import Data.Text (Text, pack, unpack)
-import Data.Monoid (mappend)
+import Data.Monoid (mappend,mconcat)
 import qualified Data.Set as Set
 import qualified Text.Blaze.Html.Renderer.Text
 import Text.Blaze.Html (toHtml)
 import Text.Blaze.Internal (preEscapedString)
+import Text.Blaze
 
-specs = describe "hamlet"
-  [ it "empty" caseEmpty
-  , it "static" caseStatic
-  , it "tag" caseTag
-  , it "var" caseVar
-  , it "var chain " caseVarChain
-  , it "url" caseUrl
-  , it "url chain " caseUrlChain
-  , it "embed" caseEmbed
-  , it "embed chain " caseEmbedChain
-  , it "if" caseIf
-  , it "if chain " caseIfChain
-  , it "else" caseElse
-  , it "else chain " caseElseChain
-  , it "elseif" caseElseIf
-  , it "elseif chain " caseElseIfChain
-  , it "list" caseList
-  , it "list chain" caseListChain
-  , it "with" caseWith
-  , it "with multi" caseWithMulti
-  , it "with chain" caseWithChain
-  , it "with comma string" caseWithCommaString
-  , it "with multi scope" caseWithMultiBindingScope
-  , it "script not empty" caseScriptNotEmpty
-  , it "meta empty" caseMetaEmpty
-  , it "input empty" caseInputEmpty
-  , it "multiple classes" caseMultiClass
-  , it "attrib order" caseAttribOrder
-  , it "nothing" caseNothing
-  , it "nothing chain " caseNothingChain
-  , it "just" caseJust
-  , it "just chain " caseJustChain
-  , it "constructor" caseConstructor
-  , it "url + params" caseUrlParams
-  , it "escape" caseEscape
-  , it "empty statement list" caseEmptyStatementList
-  , it "attribute conditionals" caseAttribCond
-  , it "non-ascii" caseNonAscii
-  , it "maybe function" caseMaybeFunction
-  , it "trailing dollar sign" caseTrailingDollarSign
-  , it "non leading percent sign" caseNonLeadingPercent
-  , it "quoted attributes" caseQuotedAttribs
-  , it "spaced derefs" caseSpacedDerefs
-  , it "attrib vars" caseAttribVars
-  , it "strings and html" caseStringsAndHtml
-  , it "nesting" caseNesting
-  , it "trailing space" caseTrailingSpace
-  , it "currency symbols" caseCurrency
-  , it "external" caseExternal
-  , it "parens" caseParens
-  , it "hamlet literals" caseHamletLiterals
-  , it "hamlet' and xhamlet'" caseHamlet'
+spec = do
+  describe "hamlet" $ do
+    it "empty" caseEmpty
+    it "static" caseStatic
+    it "tag" caseTag
+    it "var" caseVar
+    it "var chain " caseVarChain
+    it "url" caseUrl
+    it "url chain " caseUrlChain
+    it "embed" caseEmbed
+    it "embed chain " caseEmbedChain
+    it "if" caseIf
+    it "if chain " caseIfChain
+    it "else" caseElse
+    it "else chain " caseElseChain
+    it "elseif" caseElseIf
+    it "elseif chain " caseElseIfChain
+    it "list" caseList
+    it "list chain" caseListChain
+    it "with" caseWith
+    it "with multi" caseWithMulti
+    it "with chain" caseWithChain
+    it "with comma string" caseWithCommaString
+    it "with multi scope" caseWithMultiBindingScope
+    it "script not empty" caseScriptNotEmpty
+    it "meta empty" caseMetaEmpty
+    it "input empty" caseInputEmpty
+    it "multiple classes" caseMultiClass
+    it "attrib order" caseAttribOrder
+    it "nothing" caseNothing
+    it "nothing chain " caseNothingChain
+    it "just" caseJust
+    it "just chain " caseJustChain
+    it "constructor" caseConstructor
+    it "url + params" caseUrlParams
+    it "escape" caseEscape
+    it "empty statement list" caseEmptyStatementList
+    it "attribute conditionals" caseAttribCond
+    it "non-ascii" caseNonAscii
+    it "maybe function" caseMaybeFunction
+    it "trailing dollar sign" caseTrailingDollarSign
+    it "non leading percent sign" caseNonLeadingPercent
+    it "quoted attributes" caseQuotedAttribs
+    it "spaced derefs" caseSpacedDerefs
+    it "attrib vars" caseAttribVars
+    it "strings and html" caseStringsAndHtml
+    it "nesting" caseNesting
+    it "trailing space" caseTrailingSpace
+    it "currency symbols" caseCurrency
+    it "external" caseExternal
+    it "parens" caseParens
+    it "hamlet literals" caseHamletLiterals
+    it "hamlet' and xhamlet'" caseHamlet'
+    it "hamlet tuple" caseTuple
 
 
 
-  , it "comments" $ do
+    it "comments" $ do
     -- FIXME reconsider Hamlet comment syntax?
-    helper "" [hamlet|$# this is a comment
+      helper "" [hamlet|$# this is a comment
 $# another comment
 $#a third one|]
 
 
-  , it "ignores a blank line" $ do
-    helper "<p>foo</p>" [hamlet|
+    it "ignores a blank line" $ do
+      helper "<p>foo</p>\n" [hamlet|
 <p>
 
     foo
@@ -94,19 +96,22 @@ $#a third one|]
 
 
 
-  , it "hamlet angle bracket syntax" $
+    it "angle bracket syntax" $
       helper "<p class=\"foo\" height=\"100\"><span id=\"bar\" width=\"50\">HELLO</span></p>"
         [hamlet|
+$newline never
 <p.foo height="100">
     <span #bar width=50>HELLO
 |]
 
 
 
-  , it "hamlet module names" $
-    let foo = "foo" in
+    it "hamlet module names" $ do
+      let foo = "foo"
       helper "oof oof 3.14 -5"
-        [hamlet|#{Data.List.reverse foo} #
+        [hamlet|
+$newline never
+#{Data.List.reverse foo} #
 #{L.reverse foo} #
 #{show 3.14} #{show -5}|]
 
@@ -114,27 +119,27 @@ $#a third one|]
 
 
 
-  , it "single dollar at and caret" $ do
-    helper "$@^" [hamlet|\$@^|]
+    it "single dollar at and caret" $ do
+      helper "$@^" [hamlet|\$@^|]
 
-    helper "#{@{^{" [hamlet|#\{@\{^\{|]
+      helper "#{@{^{" [hamlet|#\{@\{^\{|]
 
 
-  , it "dollar operator" $ do
-    let val = (1, (2, 3))
-    helper "2" [hamlet|#{ show $ fst $ snd val }|]
-    helper "2" [hamlet|#{ show $ fst $ snd $ val}|]
+    it "dollar operator" $ do
+      let val = (1, (2, 3))
+      helper "2" [hamlet|#{ show $ fst $ snd val }|]
+      helper "2" [hamlet|#{ show $ fst $ snd $ val}|]
 
 
-  , it "in a row" $ do
-    helper "1" [hamlet|#{ show $ const 1 2 }|]
+    it "in a row" $ do
+      helper "1" [hamlet|#{ show $ const 1 2 }|]
 
 
-  , it "embedded slash" $ do
-    helper "///" [hamlet|///|]
+    it "embedded slash" $ do
+      helper "///" [hamlet|///|]
 
 {- compile-time error
-  , it "tag with slash" $ do
+    it "tag with slash" $ do
     helper "" [hamlet|
 <p>
   Text
@@ -142,21 +147,22 @@ $#a third one|]
 |]
 -}
 
-  , it "string literals" $ do
-    helper "string" [hamlet|#{"string"}|]
-    helper "string" [hamlet|#{id "string"}|]
-    helper "gnirts" [hamlet|#{L.reverse $ id "string"}|]
-    helper "str&quot;ing" [hamlet|#{"str\"ing"}|]
-    helper "str&lt;ing" [hamlet|#{"str<ing"}|]
+    it "string literals" $ do
+      helper "string" [hamlet|#{"string"}|]
+      helper "string" [hamlet|#{id "string"}|]
+      helper "gnirts" [hamlet|#{L.reverse $ id "string"}|]
+      helper "str&quot;ing" [hamlet|#{"str\"ing"}|]
+      helper "str&lt;ing" [hamlet|#{"str<ing"}|]
 
 
-  , it "interpolated operators" $ do
-    helper "3" [hamlet|#{show $ (+) 1 2}|]
-    helper "6" [hamlet|#{show $ sum $ (:) 1 ((:) 2 $ return 3)}|]
+    it "interpolated operators" $ do
+      helper "3" [hamlet|#{show $ (+) 1 2}|]
+      helper "6" [hamlet|#{show $ sum $ (:) 1 ((:) 2 $ return 3)}|]
 
 
-  , it "HTML comments" $ do
-    helper "<p>1</p><p>2 not ignored</p>" [hamlet|
+    it "HTML comments" $ do
+      helper "<p>1</p><p>2 not ignored</p>" [hamlet|
+$newline never
 <p>1
 <!-- ignored comment -->
 <p>
@@ -166,11 +172,11 @@ $#a third one|]
 
 
 
-  , it "nested maybes" $ do
-    let muser = Just "User" :: Maybe String
-        mprof = Nothing :: Maybe Int
-        m3 = Nothing :: Maybe String
-    helper "justnothing" [hamlet|
+    it "nested maybes" $ do
+      let muser = Just "User" :: Maybe String
+          mprof = Nothing :: Maybe Int
+          m3 = Nothing :: Maybe String
+      helper "justnothing" [hamlet|
 $maybe user <- muser
     $maybe profile <- mprof
         First two are Just
@@ -191,20 +197,20 @@ $nothing
 
 
 
-  , it "conditional class" $ do
-      helper "<p class=\"current\"></p>" 
+    it "conditional class" $ do
+      helper "<p class=\"current\"></p>\n"
         [hamlet|<p :False:.ignored :True:.current>|]
 
-      helper "<p class=\"1 3 2 4\"></p>"
+      helper "<p class=\"1 3 2 4\"></p>\n"
         [hamlet|<p :True:.1 :True:class=2 :False:.a :False:class=b .3 class=4>|]
 
-      helper "<p class=\"foo bar baz\"></p>"
+      helper "<p class=\"foo bar baz\"></p>\n"
         [hamlet|<p class=foo class=bar class=baz>|]
 
 
 
 
-  , it "forall on Foldable" $ do
+    it "forall on Foldable" $ do
       let set = Set.fromList [1..5 :: Int]
       helper "12345" [hamlet|
 $forall x <- set
@@ -213,74 +219,78 @@ $forall x <- set
 
 
 
-  , it "non-poly HTML" $ do
-      helperHtml "<h1>HELLO WORLD</h1>" [shamlet|
+    it "non-poly HTML" $ do
+      helperHtml "<h1>HELLO WORLD</h1>\n" [shamlet|
   <h1>HELLO WORLD
   |]
-      helperHtml "<h1>HELLO WORLD</h1>" $(shamletFile "test/hamlets/nonpolyhtml.hamlet")
+      helperHtml "<h1>HELLO WORLD</h1>\n" $(shamletFile "test/hamlets/nonpolyhtml.hamlet")
 
 
-  , it "non-poly Hamlet" $ do
+    it "non-poly Hamlet" $ do
       let embed = [hamlet|<p>EMBEDDED|]
-      helper "<h1>url</h1><p>EMBEDDED</p>" [hamlet|
+      helper "<h1>url</h1>\n<p>EMBEDDED</p>\n" [hamlet|
   <h1>@{Home}
   ^{embed}
   |]
-      helper "<h1>url</h1>" $(hamletFile "test/hamlets/nonpolyhamlet.hamlet")
+      helper "<h1>url</h1>\n" $(hamletFile "test/hamlets/nonpolyhamlet.hamlet")
 
-  , it "non-poly IHamlet" $ do
+    it "non-poly IHamlet" $ do
       let embed = [ihamlet|<p>EMBEDDED|]
-      ihelper "<h1>Adios</h1><p>EMBEDDED</p>" [ihamlet|
+      ihelper "<h1>Adios</h1>\n<p>EMBEDDED</p>\n" [ihamlet|
   <h1>_{Goodbye}
   ^{embed}
   |]
-      ihelper "<h1>Hola</h1>" $(ihamletFile "test/hamlets/nonpolyihamlet.hamlet")
+      ihelper "<h1>Hola</h1>\n" $(ihamletFile "test/hamlets/nonpolyihamlet.hamlet")
 
-  , it "pattern-match tuples: forall" $ do
+    it "pattern-match tuples: forall" $ do
       let people = [("Michael", 26), ("Miriam", 25)]
       helper "<dl><dt>Michael</dt><dd>26</dd><dt>Miriam</dt><dd>25</dd></dl>" [hamlet|
+$newline never
 <dl>
     $forall (name, age) <- people
         <dt>#{name}
         <dd>#{show age}
 |]
-  , it "pattern-match tuples: maybe" $ do
+    it "pattern-match tuples: maybe" $ do
       let people = Just ("Michael", 26)
       helper "<dl><dt>Michael</dt><dd>26</dd></dl>" [hamlet|
+$newline never
 <dl>
     $maybe (name, age) <- people
         <dt>#{name}
         <dd>#{show age}
 |]
-  , it "pattern-match tuples: with" $ do
+    it "pattern-match tuples: with" $ do
       let people = ("Michael", 26)
       helper "<dl><dt>Michael</dt><dd>26</dd></dl>" [hamlet|
+$newline never
 <dl>
     $with (name, age) <- people
         <dt>#{name}
         <dd>#{show age}
 |]
-  , it "list syntax for interpolation" $ do
+    it "list syntax for interpolation" $ do
       helper "<ul><li>1</li><li>2</li><li>3</li></ul>" [hamlet|
+$newline never
 <ul>
     $forall num <- [1, 2, 3]
         <li>#{show num}
 |]
-{-
-  , it "infix operators" $
+    it "infix operators" $
       helper "5" [hamlet|#{show $ (4 + 5) - (2 + 2)}|]
-  , it "infix operators with parens" $
-      helper "5" [hamlet|#{show (2 + 3)}|]
-      -}
-  , it "doctypes" $ helper "<!DOCTYPE html>\n<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n" [hamlet|
+    it "infix operators with parens" $
+      helper "5" [hamlet|#{show ((+) 1 1 + 3)}|]
+    it "doctypes" $ helper "<!DOCTYPE html>\n<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n" [hamlet|
+$newline never
 $doctype 5
 $doctype strict
 |]
 
-  , it "case on Maybe" $
+    it "case on Maybe" $
       let nothing  = Nothing
           justTrue = Just True
       in helper "<br><br><br><br>" [hamlet|
+$newline never
 $case nothing
     $of Just val
     $of Nothing
@@ -301,10 +311,11 @@ $case Nothing
         <br>
 |]
 
-  , it "case on Url" $
+    it "case on Url" $
       let url1 = Home
           url2 = Sub SubUrl
-      in helper "<br><br>" [hamlet|
+      in helper "<br>\n<br>\n" [hamlet|
+$newline always
 $case url1
     $of Home
         <br>
@@ -317,45 +328,108 @@ $case url2
     $of Home
 |]
 
-  , it "pattern-match constructors: forall" $ do
+    it "pattern-match constructors: forall" $ do
       let people = [Pair "Michael" 26, Pair "Miriam" 25]
       helper "<dl><dt>Michael</dt><dd>26</dd><dt>Miriam</dt><dd>25</dd></dl>" [hamlet|
+$newline text
 <dl>
     $forall Pair name age <- people
         <dt>#{name}
         <dd>#{show age}
 |]
-  , it "pattern-match constructors: maybe" $ do
+    it "pattern-match constructors: maybe" $ do
       let people = Just $ Pair "Michael" 26
       helper "<dl><dt>Michael</dt><dd>26</dd></dl>" [hamlet|
+$newline text
 <dl>
     $maybe Pair name age <- people
         <dt>#{name}
         <dd>#{show age}
 |]
-  , it "pattern-match constructors: with" $ do
+    it "pattern-match constructors: with" $ do
       let people = Pair "Michael" 26
       helper "<dl><dt>Michael</dt><dd>26</dd></dl>" [hamlet|
+$newline text
 <dl>
     $with Pair name age <- people
         <dt>#{name}
         <dd>#{show age}
 |]
 
-  , it "multiline tags" $ helper
-      "<foo bar=\"baz\" bin=\"bin\">content</foo>" [hamlet|
+    it "multiline tags" $ helper
+      "<foo bar=\"baz\" bin=\"bin\">content</foo>\n" [hamlet|
 <foo bar=baz
      bin=bin>content
 |]
-  , let attrs = [("bar", "baz"), ("bin", "<>\"&")]
-     in it "*{...} attributes" $ helper
-      "<foo bar=\"baz\" bin=\"&lt;&gt;&quot;&amp;\">content</foo>" [hamlet|
+    it "*{...} attributes" $
+      let attrs = [("bar", "baz"), ("bin", "<>\"&")] in helper
+      "<foo bar=\"baz\" bin=\"&lt;&gt;&quot;&amp;\">content</foo>\n" [hamlet|
 <foo *{attrs}>content
 |]
-  , it "blank attr values" $ helper
-      "<foo bar=\"\" baz bin=\"\"></foo>"
+    it "blank attr values" $ helper
+      "<foo bar=\"\" baz bin=\"\"></foo>\n"
       [hamlet|<foo bar="" baz bin=>|]
-  ]
+    it "greater than in attr" $ helper
+      "<button data-bind=\"enable: someFunction() > 5\">hello</button>\n"
+      [hamlet|<button data-bind="enable: someFunction() > 5">hello|]
+    it "normal doctype" $ helper
+      "<!DOCTYPE html>\n"
+      [hamlet|<!DOCTYPE html>|]
+    it "newline style" $ helper
+      "<p>foo</p>\n<pre>bar\nbaz\nbin</pre>\n"
+      [hamlet|
+$newline always
+<p>foo
+<pre>
+    bar
+    baz
+    bin
+|]
+    it "avoid newlines" $ helper
+      "<p>foo</p><pre>barbazbin</pre>"
+      [hamlet|
+$newline always
+<p>foo#
+<pre>#
+    bar#
+    baz#
+    bin#
+|]
+    it "manual linebreaks" $ helper
+      "<p>foo</p><pre>bar\nbaz\nbin</pre>"
+      [hamlet|
+$newline never
+<p>foo
+<pre>
+    bar
+    \
+    baz
+    \
+    bin
+|]
+    it "indented newline" $ helper
+      "<p>foo</p><pre>bar\nbaz\nbin</pre>"
+      [hamlet|
+    $newline never
+    <p>foo
+    <pre>
+        bar
+        \
+        baz
+        \
+        bin
+|]
+    it "case underscore" $
+        let num = 3
+         in helper "<p>Many</p>\n" [hamlet|
+$case num
+    $of 1
+        <p>1
+    $of 2
+        <p>2
+    $of _
+        <p>Many
+|]
 
 data Pair = Pair String Int
 
@@ -442,10 +516,12 @@ caseStatic = helper "some static content" [hamlet|some static content|]
 caseTag :: Assertion
 caseTag = do
     helper "<p class=\"foo\"><div id=\"bar\">baz</div></p>" [hamlet|
+$newline text
 <p .foo>
   <#bar>baz
 |]
     helper "<p class=\"foo.bar\"><div id=\"bar\">baz</div></p>" [hamlet|
+$newline text
 <p class=foo.bar>
   <#bar>baz
 |]
@@ -577,24 +653,24 @@ $with n <- " , something", y <- n
 |]
 
 caseScriptNotEmpty :: Assertion
-caseScriptNotEmpty = helper "<script></script>" [hamlet|<script>|]
+caseScriptNotEmpty = helper "<script></script>\n" [hamlet|<script>|]
 
 caseMetaEmpty :: Assertion
 caseMetaEmpty = do
-    helper "<meta>" [hamlet|<meta>|]
-    helper "<meta/>" [xhamlet|<meta>|]
+    helper "<meta>\n" [hamlet|<meta>|]
+    helper "<meta/>\n" [xhamlet|<meta>|]
 
 caseInputEmpty :: Assertion
 caseInputEmpty = do
-    helper "<input>" [hamlet|<input>|]
-    helper "<input/>" [xhamlet|<input>|]
+    helper "<input>\n" [hamlet|<input>|]
+    helper "<input/>\n" [xhamlet|<input>|]
 
 caseMultiClass :: Assertion
-caseMultiClass = helper "<div class=\"foo bar\"></div>" [hamlet|<.foo.bar>|]
+caseMultiClass = helper "<div class=\"foo bar\"></div>\n" [hamlet|<.foo.bar>|]
 
 caseAttribOrder :: Assertion
 caseAttribOrder =
-    helper "<meta 1 2 3>" [hamlet|<meta 1 2 3>|]
+    helper "<meta 1 2 3>\n" [hamlet|<meta 1 2 3>|]
 
 caseNothing :: Assertion
 caseNothing = do
@@ -641,6 +717,7 @@ caseUrlParams = do
 caseEscape :: Assertion
 caseEscape = do
     helper "#this is raw\n " [hamlet|
+$newline never
 \#this is raw
 \
 \ 
@@ -656,16 +733,16 @@ caseEmptyStatementList = do
 
 caseAttribCond :: Assertion
 caseAttribCond = do
-    helper "<select></select>" [hamlet|<select :False:selected>|]
-    helper "<select selected></select>" [hamlet|<select :True:selected>|]
-    helper "<meta var=\"foo:bar\">" [hamlet|<meta var=foo:bar>|]
-    helper "<select selected></select>"
+    helper "<select></select>\n" [hamlet|<select :False:selected>|]
+    helper "<select selected></select>\n" [hamlet|<select :True:selected>|]
+    helper "<meta var=\"foo:bar\">\n" [hamlet|<meta var=foo:bar>|]
+    helper "<select selected></select>\n"
         [hamlet|<select :true theArg:selected>|]
 
-    helper "<select></select>" [hamlet|<select :False:selected>|]
-    helper "<select selected></select>" [hamlet|<select :True:selected>|]
-    helper "<meta var=\"foo:bar\">" [hamlet|<meta var=foo:bar>|]
-    helper "<select selected></select>"
+    helper "<select></select>\n" [hamlet|<select :False:selected>|]
+    helper "<select selected></select>\n" [hamlet|<select :True:selected>|]
+    helper "<meta var=\"foo:bar\">\n" [hamlet|<meta var=foo:bar>|]
+    helper "<select selected></select>\n"
         [hamlet|<select :true theArg:selected>|]
 
 caseNonAscii :: Assertion
@@ -681,7 +758,9 @@ $maybe x <- Just urlParams
 
 caseTrailingDollarSign :: Assertion
 caseTrailingDollarSign =
-    helper "trailing space \ndollar sign #" [hamlet|trailing space #
+    helper "trailing space \ndollar sign #" [hamlet|
+$newline never
+trailing space #
 \
 dollar sign #\
 |]
@@ -689,29 +768,31 @@ dollar sign #\
 caseNonLeadingPercent :: Assertion
 caseNonLeadingPercent =
     helper "<span style=\"height:100%\">foo</span>" [hamlet|
+$newline never
 <span style=height:100%>foo
 |]
 
 caseQuotedAttribs :: Assertion
 caseQuotedAttribs =
     helper "<input type=\"submit\" value=\"Submit response\">" [hamlet|
+$newline never
 <input type=submit value="Submit response">
 |]
 
 caseSpacedDerefs :: Assertion
 caseSpacedDerefs = do
     helper "&lt;var&gt;" [hamlet|#{var theArg}|]
-    helper "<div class=\"&lt;var&gt;\"></div>" [hamlet|<.#{var theArg}>|]
+    helper "<div class=\"&lt;var&gt;\"></div>\n" [hamlet|<.#{var theArg}>|]
 
 caseAttribVars :: Assertion
 caseAttribVars = do
-    helper "<div id=\"&lt;var&gt;\"></div>" [hamlet|<##{var theArg}>|]
-    helper "<div class=\"&lt;var&gt;\"></div>" [hamlet|<.#{var theArg}>|]
-    helper "<div f=\"&lt;var&gt;\"></div>" [hamlet|< f=#{var theArg}>|]
+    helper "<div id=\"&lt;var&gt;\"></div>\n" [hamlet|<##{var theArg}>|]
+    helper "<div class=\"&lt;var&gt;\"></div>\n" [hamlet|<.#{var theArg}>|]
+    helper "<div f=\"&lt;var&gt;\"></div>\n" [hamlet|< f=#{var theArg}>|]
 
-    helper "<div id=\"&lt;var&gt;\"></div>" [hamlet|<##{var theArg}>|]
-    helper "<div class=\"&lt;var&gt;\"></div>" [hamlet|<.#{var theArg}>|]
-    helper "<div f=\"&lt;var&gt;\"></div>" [hamlet|< f=#{var theArg}>|]
+    helper "<div id=\"&lt;var&gt;\"></div>\n" [hamlet|<##{var theArg}>|]
+    helper "<div class=\"&lt;var&gt;\"></div>\n" [hamlet|<.#{var theArg}>|]
+    helper "<div f=\"&lt;var&gt;\"></div>\n" [hamlet|< f=#{var theArg}>|]
 
 caseStringsAndHtml :: Assertion
 caseStringsAndHtml = do
@@ -724,6 +805,7 @@ caseNesting = do
     helper
       "<table><tbody><tr><td>1</td></tr><tr><td>2</td></tr></tbody></table>"
       [hamlet|
+$newline never
 <table>
   <tbody>
     $forall user <- users
@@ -738,6 +820,7 @@ caseNesting = do
           , "</select>"
           ])
         [hamlet|
+$newline never
 <select #"#{name}" name=#{name}>
     <option :isBoolBlank val:selected>
     <option value=true :isBoolTrue val:selected>Yes
@@ -763,8 +846,8 @@ caseCurrency =
 
 caseExternal :: Assertion
 caseExternal = do
-    helper "foo<br>" $(hamletFile "test/hamlets/external.hamlet")
-    helper "foo<br/>" $(xhamletFile "test/hamlets/external.hamlet")
+    helper "foo\n<br>\n" $(hamletFile "test/hamlets/external.hamlet")
+    helper "foo\n<br/>\n" $(xhamletFile "test/hamlets/external.hamlet")
   where
     foo = "foo"
 
@@ -795,14 +878,14 @@ caseHamlet' :: Assertion
 caseHamlet' = do
     helper' "foo" [shamlet|foo|]
     helper' "foo" [xshamlet|foo|]
-    helper "<br>" $ const $ [shamlet|<br>|]
-    helper "<br/>" $ const $ [xshamlet|<br>|]
+    helper "<br>\n" $ const $ [shamlet|<br>|]
+    helper "<br/>\n" $ const $ [xshamlet|<br>|]
 
     -- new with generalized stuff
     helper' "foo" [shamlet|foo|]
     helper' "foo" [xshamlet|foo|]
-    helper "<br>" $ const $ [shamlet|<br>|]
-    helper "<br/>" $ const $ [xshamlet|<br>|]
+    helper "<br>\n" $ const $ [shamlet|<br>|]
+    helper "<br/>\n" $ const $ [xshamlet|<br>|]
 
 
 instance Show Url where
@@ -829,6 +912,21 @@ $forall   x     <-   empty
 
 
 
+caseTuple :: Assertion
+caseTuple = do
+   helper "(1,1)" [hamlet| #{("1","1")}|]
+   helper "foo" [hamlet| 
+    $with (a,b) <- ("foo","bar")
+      #{a}
+   |]
+   helper "url" [hamlet| 
+    $with (a,b) <- (Home,Home)
+      @{a}
+   |]
+   helper "url" [hamlet| 
+    $with p <- (Home,[])
+      @?{p}
+   |]
 
 data Msg = Hello | Goodbye
 
@@ -839,3 +937,11 @@ ihelper res h = do
   where
     showMsg Hello = preEscapedString "Hola"
     showMsg Goodbye = preEscapedString "Adios"
+
+instance (ToMarkup a, ToMarkup b) => ToMarkup (a,b) where
+  toMarkup (a,b) = do
+    toMarkup "("
+    toMarkup a
+    toMarkup ","
+    toMarkup b
+    toMarkup ")"
